//@author: vux
//@help: template for standard shaders
//@tags: template
//@credits: 

Texture2D texture2d <string uiname="Texture";>;

SamplerState linearSampler : IMMUTABLE
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Clamp;
    AddressV = Clamp;
};
 
cbuffer cbPerDraw : register( b0 )
{
	float4x4 tVP : LAYERVIEWPROJECTION;	
};

cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
};

struct VS_IN
{
	uint vi : SV_VertexID;
	float4 PosO : POSITION;
	float4 TexCd : TEXCOORD0;

};

struct vs2ps
{
    float4 PosWVP: SV_Position;
	float4 Color: Color;
    float4 TexCd: TEXCOORD0;
};

struct CustomVertex
{
	float3 Position;
	float pad0;
	float4 Color;
};

StructuredBuffer<CustomVertex> VertexBufferStructured;
Buffer<float> VertexBuffer;

CustomVertex GetCustomVertexRaw(uint index)
{
	const uint floatCount = 8;
	uint i = index * floatCount;
	CustomVertex result;
	result.Position = float3(VertexBuffer[i], VertexBuffer[i+1], VertexBuffer[i+2]);
	result.Color = float4(VertexBuffer[i+4], VertexBuffer[i+5], VertexBuffer[i+6], VertexBuffer[i+7]);
	return result;
}

vs2ps VSRaw(VS_IN input)
{
    vs2ps output;
	CustomVertex v = GetCustomVertexRaw(input.vi);
    output.PosWVP  = mul(float4(v.Position, 1), mul(tW,tVP));
	output.Color = v.Color;
    output.TexCd = input.TexCd;
    return output;
}

vs2ps VS(VS_IN input)
{
    vs2ps output;
	CustomVertex v = VertexBufferStructured[input.vi];
    output.PosWVP  = mul(float4(v.Position, 1), mul(tW,tVP));
	output.Color = v.Color;
    output.TexCd = input.TexCd;
    return output;
}

float4 PS(vs2ps In): SV_Target
{
    float4 col = In.Color;
    return col;
}

technique10 ConstantRaw
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VSRaw() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}

technique10 ConstantStructured
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}




